package builtins_test

import (
	"bytes"
	"context"
	"github.com/stretchr/testify/assert"
	"testing"
	"ucl.lmika.dev/ucl"
	"ucl.lmika.dev/ucl/builtins"
)

func TestLog_Puts(t *testing.T) {
	tests := []struct {
		desc    string
		eval    string
		want    any
		wantErr bool
		wantLog string
	}{
		{desc: "log 1", eval: `log:puts "bad stuff here"`, wantLog: "bad stuff here\n"},
		{desc: "log 2", eval: `log:puts "bad " 123 " here " [1 2 3]`, wantLog: "bad 123 here [1 2 3]\n"},
		{desc: "log 3", eval: `log:puts`, wantLog: "\n"},
	}

	for _, tt := range tests {
		t.Run(tt.desc, func(t *testing.T) {
			var logMessage bytes.Buffer

			inst := ucl.New(
				ucl.WithModule(builtins.Log(func(s string) error {
					logMessage.WriteString(s)
					logMessage.WriteByte('\n')
					return nil
				})),
			)

			res, err := inst.Eval(context.Background(), tt.eval)
			if tt.wantErr {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.want, res)
			}
			assert.Equal(t, tt.wantLog, logMessage.String())
		})
	}
}
